# !/usr/bin/python
# coding=utf-8
from uitk.slots import Slots



class Subdivision(Slots):
	'''
	'''
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		'''
		'''
	

	def draggable_header(self, state=None):
		'''Context menu
		'''
		dh = self.sb.subdivision.draggable_header

